@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 """A module for miscellaneous useful bits and bobs that don't
 obviously belong anywhere else. If you spot a better home for
@@ -19,15 +14,37 @@ anything that lives here, please move it."""
 
 import array
 import sys
+from random import Random
+from typing import (
+    Callable,
+    Dict,
+    Generic,
+    Iterable,
+    Iterator,
+    List,
+    Optional,
+    Sequence,
+    Tuple,
+    TypeVar,
+    Union,
+    overload,
+)
+
+ARRAY_CODES = ["B", "H", "I", "L", "Q", "O"]
 
 
-def array_or_list(code, contents):
+def array_or_list(
+    code: str, contents: Iterable[int]
+) -> "Union[List[int], array.ArrayType[int]]":
     if code == "O":
         return list(contents)
     return array.array(code, contents)
 
 
-def replace_all(buffer, replacements):
+def replace_all(
+    buffer: Sequence[int],
+    replacements: Iterable[Tuple[int, int, Sequence[int]]],
+) -> bytes:
     """Substitute multiple replacement values into a buffer.
 
     Replacements is a list of (start, end, value) triples.
@@ -46,11 +63,10 @@ def replace_all(buffer, replacements):
     return bytes(result)
 
 
-ARRAY_CODES = ["B", "H", "I", "L", "Q", "O"]
 NEXT_ARRAY_CODE = dict(zip(ARRAY_CODES, ARRAY_CODES[1:]))
 
 
-class IntList:
+class IntList(Sequence[int]):
     """Class for storing a list of non-negative integers compactly.
 
     We store them as the smallest size integer array we can get
@@ -60,67 +76,78 @@ class IntList:
 
     __slots__ = ("__underlying",)
 
-    def __init__(self, values=()):
+    __underlying: "Union[List[int], array.ArrayType[int]]"
+
+    def __init__(self, values: Sequence[int] = ()):
         for code in ARRAY_CODES:
             try:
-                self.__underlying = array_or_list(code, values)
+                underlying = array_or_list(code, values)
                 break
             except OverflowError:
                 pass
         else:  # pragma: no cover
-            raise AssertionError("Could not create storage for %r" % (values,))
-        if isinstance(self.__underlying, list):
-            for v in self.__underlying:
-                if v < 0 or not isinstance(v, int):
-                    raise ValueError("Could not create IntList for %r" % (values,))
+            raise AssertionError(f"Could not create storage for {values!r}")
+        if isinstance(underlying, list):
+            for v in underlying:
+                if not isinstance(v, int) or v < 0:
+                    raise ValueError(f"Could not create IntList for {values!r}")
+        self.__underlying = underlying
 
     @classmethod
-    def of_length(self, n):
-        return IntList(array_or_list("B", [0]) * n)
+    def of_length(cls, n: int) -> "IntList":
+        return cls(array_or_list("B", [0]) * n)
 
-    def count(self, n):
-        return self.__underlying.count(n)
+    def count(self, value: int) -> int:
+        return self.__underlying.count(value)
 
     def __repr__(self):
-        return "IntList(%r)" % (list(self),)
+        return f"IntList({list(self.__underlying)!r})"
 
     def __len__(self):
         return len(self.__underlying)
 
-    def __getitem__(self, i):
+    @overload
+    def __getitem__(self, i: int) -> int:
+        ...  # pragma: no cover
+
+    @overload
+    def __getitem__(self, i: slice) -> "IntList":
+        ...  # pragma: no cover
+
+    def __getitem__(self, i: Union[int, slice]) -> "Union[int, IntList]":
         if isinstance(i, slice):
             return IntList(self.__underlying[i])
         return self.__underlying[i]
 
-    def __delitem__(self, i):
+    def __delitem__(self, i: int) -> None:
         del self.__underlying[i]
 
-    def insert(self, i, v):
+    def insert(self, i: int, v: int) -> None:
         self.__underlying.insert(i, v)
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[int]:
         return iter(self.__underlying)
 
-    def __eq__(self, other):
+    def __eq__(self, other: object) -> bool:
         if self is other:
             return True
         if not isinstance(other, IntList):
             return NotImplemented
         return self.__underlying == other.__underlying
 
-    def __ne__(self, other):
+    def __ne__(self, other: object) -> bool:
         if self is other:
             return False
         if not isinstance(other, IntList):
             return NotImplemented
         return self.__underlying != other.__underlying
 
-    def append(self, n):
+    def append(self, n: int) -> None:
         i = len(self)
         self.__underlying.append(0)
         self[i] = n
 
-    def __setitem__(self, i, n):
+    def __setitem__(self, i: int, n: int) -> None:
         while True:
             try:
                 self.__underlying[i] = n
@@ -129,16 +156,17 @@ class IntList:
                 assert n > 0
                 self.__upgrade()
 
-    def extend(self, ls):
+    def extend(self, ls: Iterable[int]) -> None:
         for n in ls:
             self.append(n)
 
-    def __upgrade(self):
+    def __upgrade(self) -> None:
+        assert isinstance(self.__underlying, array.array)
         code = NEXT_ARRAY_CODE[self.__underlying.typecode]
         self.__underlying = array_or_list(code, self.__underlying)
 
 
-def binary_search(lo, hi, f):
+def binary_search(lo: int, hi: int, f: Callable[[int], bool]) -> int:
     """Binary searches in [lo , hi) to find
     n such that f(n) == f(lo) but f(n + 1) != f(lo).
     It is implicitly assumed and will not be checked
@@ -156,11 +184,14 @@ def binary_search(lo, hi, f):
     return lo
 
 
-def uniform(random, n):
+def uniform(random: Random, n: int) -> bytes:
     """Returns a bytestring of length n, distributed uniformly at random."""
     return random.getrandbits(n * 8).to_bytes(n, "big")
 
 
+T = TypeVar("T")
+
+
 class LazySequenceCopy:
     """A "copy" of a sequence that works by inserting a mask in front
     of the underlying sequence, so that you can mutate it without changing
@@ -168,15 +199,17 @@ class LazySequenceCopy:
     in O(1) time. The full list API is not supported yet but there's no reason
     in principle it couldn't be."""
 
-    def __init__(self, values):
+    __mask: Optional[Dict[int, int]]
+
+    def __init__(self, values: Sequence[int]):
         self.__values = values
         self.__len = len(values)
         self.__mask = None
 
-    def __len__(self):
+    def __len__(self) -> int:
         return self.__len
 
-    def pop(self):
+    def pop(self) -> int:
         if len(self) == 0:
             raise IndexError("Cannot pop from empty list")
         result = self[-1]
@@ -185,7 +218,7 @@ class LazySequenceCopy:
             self.__mask.pop(self.__len, None)
         return result
 
-    def __getitem__(self, i):
+    def __getitem__(self, i: int) -> int:
         i = self.__check_index(i)
         default = self.__values[i]
         if self.__mask is None:
@@ -193,36 +226,36 @@ class LazySequenceCopy:
         else:
             return self.__mask.get(i, default)
 
-    def __setitem__(self, i, v):
+    def __setitem__(self, i: int, v: int) -> None:
         i = self.__check_index(i)
         if self.__mask is None:
             self.__mask = {}
         self.__mask[i] = v
 
-    def __check_index(self, i):
+    def __check_index(self, i: int) -> int:
         n = len(self)
         if i < -n or i >= n:
-            raise IndexError("Index %d out of range [0, %d)" % (i, n))
+            raise IndexError(f"Index {i} out of range [0, {n})")
         if i < 0:
             i += n
         assert 0 <= i < n
         return i
 
 
-def clamp(lower, value, upper):
+def clamp(lower: int, value: int, upper: int) -> int:
     """Given a value and lower/upper bounds, 'clamp' the value so that
     it satisfies lower <= value <= upper."""
     return max(lower, min(value, upper))
 
 
-def swap(ls, i, j):
+def swap(ls: LazySequenceCopy, i: int, j: int) -> None:
     """Swap the elements ls[i], ls[j]."""
     if i == j:
         return
     ls[i], ls[j] = ls[j], ls[i]
 
 
-def stack_depth_of_caller():
+def stack_depth_of_caller() -> int:
     """Get stack size for caller's frame.
 
     From https://stackoverflow.com/a/47956089/9297601 , this is a simple
@@ -233,12 +266,12 @@ def stack_depth_of_caller():
     frame = sys._getframe(2)
     size = 1
     while frame:
-        frame = frame.f_back
+        frame = frame.f_back  # type: ignore[assignment]
         size += 1
     return size
 
 
-def find_integer(f):
+def find_integer(f: Callable[[int], bool]) -> int:
     """Finds a (hopefully large) integer such that f(n) is True and f(n + 1) is
     False.
 
@@ -276,7 +309,7 @@ def find_integer(f):
     return lo
 
 
-def pop_random(random, seq):
+def pop_random(random: Random, seq: LazySequenceCopy) -> int:
     """Remove and return a random element of seq. This runs in O(1) but leaves
     the sequence in an arbitrary order."""
     i = random.randrange(0, len(seq))
@@ -288,7 +321,7 @@ class NotFound(Exception):
     pass
 
 
-class SelfOrganisingList:
+class SelfOrganisingList(Generic[T]):
     """A self-organising list with the move-to-front heuristic.
 
     A self-organising list is a collection which we want to retrieve items
@@ -305,17 +338,17 @@ class SelfOrganisingList:
 
     """
 
-    def __init__(self, values=()):
+    def __init__(self, values: Iterable[T] = ()) -> None:
         self.__values = list(values)
 
-    def __repr__(self):
-        return "SelfOrganisingList(%r)" % (self.__values,)
+    def __repr__(self) -> str:
+        return f"SelfOrganisingList({self.__values!r})"
 
-    def add(self, value):
+    def add(self, value: T) -> None:
         """Add a value to this list."""
         self.__values.append(value)
 
-    def find(self, condition):
+    def find(self, condition: Callable[[T], bool]) -> T:
         """Returns some value in this list such that ``condition(value)`` is
         True. If no such value exists raises ``NotFound``."""
         for i in range(len(self.__values) - 1, -1, -1):
