@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 """This module contains various provisional APIs and strategies.
 
@@ -23,8 +18,8 @@ definitions it links to.  If not, report the bug!
 """
 # https://tools.ietf.org/html/rfc3696
 
-import os.path
 import string
+from importlib import resources
 
 from hypothesis import strategies as st
 from hypothesis.errors import InvalidArgument
@@ -37,18 +32,18 @@ FRAGMENT_SAFE_CHARACTERS = URL_SAFE_CHARACTERS | {"?", "/"}
 
 # This file is sourced from http://data.iana.org/TLD/tlds-alpha-by-domain.txt
 # The file contains additional information about the date that it was last updated.
-try:
-    from importlib.resources import read_text  # type: ignore
-except ImportError:
-    # If we don't have importlib.resources (Python 3.7+) or the importlib_resources
-    # backport available, fall back to __file__ and hope we're on a filesystem.
-    f = os.path.join(os.path.dirname(__file__), "vendor", "tlds-alpha-by-domain.txt")
-    with open(f) as tld_file:
-        _tlds = tld_file.read().splitlines()
-else:  # pragma: no cover  # new in Python 3.7
-    _tlds = read_text("hypothesis.vendor", "tlds-alpha-by-domain.txt").splitlines()
-assert _tlds[0].startswith("#")
-TOP_LEVEL_DOMAINS = ["COM"] + sorted(_tlds[1:], key=len)
+try:  # pragma: no cover
+    traversable = resources.files("hypothesis.vendor") / "tlds-alpha-by-domain.txt"
+    _comment, *_tlds = traversable.read_text().splitlines()
+except (AttributeError, ValueError):  # pragma: no cover  # .files() was added in 3.9
+    _comment, *_tlds = resources.read_text(
+        "hypothesis.vendor", "tlds-alpha-by-domain.txt"
+    ).splitlines()
+assert _comment.startswith("#")
+
+# Remove special-use domain names from the list. For more discussion
+# see https://github.com/HypothesisWorks/hypothesis/pull/3572
+TOP_LEVEL_DOMAINS = ["COM"] + sorted((d for d in _tlds if d != "ARPA"), key=len)
 
 
 class DomainNameStrategy(st.SearchStrategy):
@@ -58,13 +53,11 @@ class DomainNameStrategy(st.SearchStrategy):
             value = maximum
         elif not isinstance(value, int):
             raise InvalidArgument(
-                "Expected integer but %s is a %s"
-                % (variable_name, type(value).__name__)
+                f"Expected integer but {variable_name} is a {type(value).__name__}"
             )
         elif not minimum <= value <= maximum:
             raise InvalidArgument(
-                "Invalid value %r < %s=%r < %r"
-                % (minimum, variable_name, value, maximum)
+                f"Invalid value {minimum!r} < {variable_name}={value!r} < {maximum!r}"
             )
         return value
 
@@ -116,15 +109,20 @@ class DomainNameStrategy(st.SearchStrategy):
                 ).map("".join)
             )
         )
+        # RFC-5890 s2.3.1 says such labels are reserved, and since we don't
+        # want to bother with xn-- punycode labels we'll exclude them all.
+        elem_st = st.from_regex(self.label_regex, fullmatch=True).filter(
+            lambda label: len(label) < 4 or label[2:4] != "--"
+        )
         # The maximum possible number of subdomains is 126,
         # 1 character subdomain + 1 '.' character, * 126 = 252,
         # with a max of 255, that leaves 3 characters for a TLD.
         # Allowing any more subdomains would not leave enough
         # characters for even the shortest possible TLDs.
-        elements = cu.many(data, min_size=1, average_size=1, max_size=126)
+        elements = cu.many(data, min_size=1, average_size=3, max_size=126)
         while elements.more():
             # Generate a new valid subdomain using the regex strategy.
-            sub_domain = data.draw(st.from_regex(self.label_regex, fullmatch=True))
+            sub_domain = data.draw(elem_st)
             if len(domain) + len(sub_domain) >= self.max_length:
                 data.stop_example(discard=True)
                 break
@@ -169,7 +167,7 @@ def urls() -> st.SearchStrategy[str]:
         return "".join(c if c in URL_SAFE_CHARACTERS else "%%%02X" % ord(c) for c in s)
 
     schemes = st.sampled_from(["http", "https"])
-    ports = st.integers(min_value=0, max_value=2 ** 16 - 1).map(":{}".format)
+    ports = st.integers(min_value=0, max_value=2**16 - 1).map(":{}".format)
     paths = st.lists(st.text(string.printable).map(url_encode)).map("/".join)
 
     return st.builds(
